/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import java.io.*;

/**
 * Provides a feed to the webpage system.
 *
 * @author  $Author: tdb $
 * @version $Id: WebFeeder.java,v 1.24 2003/02/05 16:43:45 tdb Exp $
 */
public class WebFeeder extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.24 $";
    
    /**
     * Default check period in seconds (30 minutes)
     */
    public final int DEFAULT_CHECK_PERIOD = 1800;
    
    /**
     * Delete alerts older than this in seconds, default.
     */
    public final int DEFAULT_AGE = 3600;
    
    /**
     * The default path seperator, here for convienience
     */
    private final String sep = File.separator;
    
//---STATIC METHODS---

    /**
     * Return a reference to the single class.
     * Construct it if it does not already exist, otherwise just return the reference.
     */
    public synchronized static WebFeeder getInstance() {
        if (_instance == null){
            _instance = new WebFeeder();
        }
        return _instance;
    }

//---CONSTRUCTORS---
    
    /**
     * Construct a new WebFeeder. This will also wipe out any
     * old Alerts, as these can't be carried from one session
     * until the next.
     */
    private WebFeeder() {
        // do something, or nothing.. but must be private
        // don't need to cleanup latest data
        
        // -- cleanup old alerts
        // get config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        // get file locations
        String rootPath, alertSubDir, alertFileName;
        try {
            // work out where things are
            rootPath = cp.getProperty("WebFeeder", "WebFeeder.rootPath");
            alertSubDir = cp.getProperty("WebFeeder", "WebFeeder.alertSubDir");
            File alertsDir = new File(rootPath, alertSubDir);
            if(deleteContents(alertsDir)) {
                _logger.write(this.toString(), Logger.DEBUG, "Deleted all files and directories from: "+rootPath+sep+alertSubDir);
            } else {
                _logger.write(this.toString(), Logger.WARNING, "Failed to delete all files and directories from: "+rootPath+sep+alertSubDir);
            }
            // cleanup complete
        } catch (PropertyNotFoundException e) {
            _logger.write(this.toString(), Logger.ERROR, "Failed to cleanup on construction, due to failing to get config for Alert Data: "+e);
            // just leave it at that
        }
        
        // set our name and startup
        setName("client.WebFeeder");
        start();
    }

//---PUBLIC METHODS---
    
    /**
     * Thread loop, will check at intervals for any files
     * that need to be "cleaned up". This will normally
     * be OK and FINAL alerts that have been around for longer
     * than a specified period of time.
     */
    public void run() {
        boolean running = true;
        // get config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        // loop round
        while(running) {
            // get our check period
            int checkPeriod = 0;
            try {
                checkPeriod = Integer.parseInt(cp.getProperty("WebFeeder", "WebFeeder.checkPeriod"));
            } catch (NumberFormatException e) {
                checkPeriod = DEFAULT_CHECK_PERIOD;
                _logger.write(toString(), Logger.WARNING, "Erronous WebFeeder.checkPeriod value in configuration using default of " + checkPeriod + " seconds");
            } catch (PropertyNotFoundException e) {
                checkPeriod = DEFAULT_CHECK_PERIOD;
                _logger.write(toString(), Logger.WARNING, "WebFeeder.checkPeriod value unavailable using default of " + checkPeriod + " seconds");
            }
            // wait for the check period
            try {
                Thread.sleep(checkPeriod * 1000);
            } catch (InterruptedException e) {
            }
            
            // get alerts directory
            String rootPath, alertSubDir, alertFileName;
            try {
                rootPath = cp.getProperty("WebFeeder", "WebFeeder.rootPath");
                alertSubDir = cp.getProperty("WebFeeder", "WebFeeder.alertSubDir");
                alertFileName = cp.getProperty("WebFeeder", "WebFeeder.alertFileName");
            } catch (PropertyNotFoundException e) {
                _logger.write(this.toString(), Logger.ERROR, "Failed to get config for Alert Data: "+e);
                // bail out
                continue;
            }
            
            // get the "age" barrier
            int deleteOlderThan = 0;
            try {
                deleteOlderThan = Integer.parseInt(cp.getProperty("WebFeeder", "WebFeeder.alertDeleteOlderThan"));
            } catch (NumberFormatException e) {
                deleteOlderThan = DEFAULT_AGE;
                _logger.write(toString(), Logger.WARNING, "Erronous WebFeeder.alertDeleteOlderThan value in configuration using default of " + deleteOlderThan + " seconds");
            } catch (PropertyNotFoundException e) {
                deleteOlderThan = DEFAULT_AGE;
                _logger.write(toString(), Logger.WARNING, "WebFeeder.alertDeleteOlderThan value unavailable using default of " + deleteOlderThan + " seconds");
            }
            
            // list the files and delete as appropriate
            File alertsDir = new File(rootPath, alertSubDir);
            // check it's a directory
            if(alertsDir.isDirectory()) {
                // get all the hostnames directories
                File[] contents = alertsDir.listFiles();
                for(int i=0; i < contents.length; i++) {
                    // get a single directory from the array..
                    File hostdir = contents[i];
                    // ..and check it's a directory
                    if(hostdir.isDirectory()) {
                        // if this is set, we clean files older than it
                        long deleteFiles = -1;
                        // get all the contents of that directory
                        File[] hostdirContents = hostdir.listFiles();
                        for(int j=0; j < hostdirContents.length; j++) {
                            File alertFile = hostdirContents[j];
                            // get the filename..
                            String filename = alertFile.getName();
                            // ..and see if it ends with OK or FINAL
                            if(filename.endsWith(Alert.alertLevels[0]) ||
                               filename.endsWith(Alert.alertLevels[Alert.alertLevels.length-1])) {
                                // it does end with either OK or FINAL
                                // ... so we can check it for deletion
                                long lastModified = alertFile.lastModified();
                                long age = System.currentTimeMillis() - lastModified;
                                if(age > ((long) deleteOlderThan*1000)) {
                                    // if we're on a final heartbeat, we probably want to
                                    // clean up any stale alerts left behind
                                    // by setting this flag, we'll clean them up on leaving this loop
                                    if(filename.endsWith(".HB."+Alert.alertLevels[Alert.alertLevels.length-1])) {
                                        // we do this so that delete files is set to the
                                        // latest date of a HB.FINAL. There should only be
                                        // one of them though :)
                                        if(lastModified > deleteFiles) {
                                            deleteFiles = lastModified;
                                        }
                                    }
                                    // it's also older than our age to delete older than
                                    if(!alertFile.delete()) {
                                        _logger.write(this.toString(), Logger.WARNING, "Failed to delete the following 'old' alert file: "+alertFile.getPath());
                                    }
                                }
                            }
                        }
                        // cleanup stale alerts
                        if(deleteFiles >= 0) {
                            File[] remainingHostdirContents = hostdir.listFiles();
                            for(int j=0; j < remainingHostdirContents.length; j++) {
                                File alertFile = remainingHostdirContents[j];
                                if(alertFile.lastModified() < deleteFiles) {
                                    // alert file is older than the most recent
                                    // FINAL Heartbeat alert.
                                    if(alertFile.delete()) {
                                        _logger.write(this.toString(), Logger.DEBUG, "Deleted stale alert file: "+alertFile.getPath());
                                    }
                                    else {
                                        _logger.write(this.toString(), Logger.WARNING, "Failed to delete the following 'stale' alert file: "+alertFile.getPath());
                                    }
                                }
                            }
                        }
                        // ---- RECAP ----
                        // at this point, we have cleaned up any OK or FINAL alerts
                        // that have passed our age limit. We have then cleaned up
                        // any alerts older the most recent Heartbeat FINAL alert,
                        // as these are probably stale. Any files left are valid and
                        // active alerts. We are now in a position to remove the host
                        // directory if it's empty.
                        // ---------------
                        // do a quick check to see if the directory is now empty
                        File[] newHostdirContents = hostdir.listFiles();
                        if(newHostdirContents.length == 0) {
                            // it does seem to be, try and delete it
                            // this will fail anyway if files still remain
                            if(!hostdir.delete()) {
                                _logger.write(this.toString(), Logger.WARNING, "Failed to delete the following empty host directory: "+hostdir.getPath());
                            }
                        }
                    }
                }
            }
            else {
                _logger.write(toString(), Logger.WARNING, "IO error reading alerts directory, maybe it doesn't exist? : " +rootPath+sep+alertSubDir);
            }
        }
    }
    
    /**
     * Handles an XMLPacket. This will write it out to disk
     * in an appropriate manner.
     *
     * @param packet the XMLPacket to write
     */
    public void receiveXMLPacket(XMLPacket packet) {
        String packetType = packet.getParam("packet.attributes.type");
        if(packetType == null || !packetType.equals("data")) {
            // bail out, without warning
            // this is probably a heartbeat or similar
            return;
        }
        // get config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        // get file locations
        String rootPath, latestSubDir, latestFileName;
        try {
            rootPath = cp.getProperty("WebFeeder", "WebFeeder.rootPath");
            latestSubDir = cp.getProperty("WebFeeder", "WebFeeder.latestSubDir");
            latestFileName = cp.getProperty("WebFeeder", "WebFeeder.latestFileName");
        } catch (PropertyNotFoundException e) {
            _logger.write(this.toString(), Logger.ERROR, "Failed to get config for Latest Data: "+e);
            // bail out
            return;
        }
        // get raw data
        String data = packet.printAll();
        String hostname = packet.getParam("packet.attributes.machine_name");
        // set paths
        File outDir = new File(rootPath, latestSubDir+sep+hostname);
        File outFile = new File(rootPath, latestSubDir+sep+hostname+sep+latestFileName);
        // write the data out
        writeData(outDir, outFile, data);
    }
    
    /**
     * Handles an Alert. This will write it out to disk
     * in an appropriate manner.
     *
     * @param alert the Alert object to write
     */
    public void receiveAlert(Alert alert) {
        // get config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        // get file locations
        String rootPath, alertSubDir, alertFileName;
        try {
            rootPath = cp.getProperty("WebFeeder", "WebFeeder.rootPath");
            alertSubDir = cp.getProperty("WebFeeder", "WebFeeder.alertSubDir");
            alertFileName = cp.getProperty("WebFeeder", "WebFeeder.alertFileName");
        } catch (PropertyNotFoundException e) {
            _logger.write(this.toString(), Logger.ERROR, "Failed to get config for Alert Data: "+e);
            // bail out
            return;
        }
        // get raw data
        String data = alert.printAll();
        String hostname = alert.getSource();
        // set paths
        File outDir = new File(rootPath, alertSubDir+sep+hostname);
        String destFile = alertSubDir+sep+hostname+sep+alertFileName+"."+String.valueOf(alert.getInitialAlertTime());
        File outFile;
        // check if we're at a special "end case" (OK or FINAL)
        if(alert.getLevel()==0 || alert.getLevel()==Alert.alertLevels.length-1) {
            if(alert.getAttributeName().equals("Heartbeat") && alert.getLevel()==Alert.alertLevels.length-1) {
                // new file is something like alert.nnnnnnnn.HB.FINAL
                outFile = new File(rootPath, destFile+".HB."+Alert.alertLevels[alert.getLevel()]);
            } else {
                // new file is something like alert.nnnnnnnn.OK
                outFile = new File(rootPath, destFile+"."+Alert.alertLevels[alert.getLevel()]);
            }
            File oldFile = new File(rootPath, destFile);
            if(!oldFile.renameTo(outFile)) {
                _logger.write(this.toString(), Logger.WARNING, "Failed to rename old file, "+oldFile.getPath()+" to new file, "+outFile.getPath());
            }
        } else {
            outFile = new File(rootPath, destFile);
        }
        // write the data out
        writeData(outDir, outFile, data);
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---
    
    /**
     * Attempts to write "data" to "outFile" in "outDir".
     *
     * Performs checks to create the directories, and the file.
     * Does not "return" anything to indicate failure or success.
     *
     * @param outDir the directory to put the file in
     * @param outFile the filename to put the data in
     * @param data the String of data to write
     */
    private void writeData(File outDir, File outFile, String data) {
        // try to create directory
        if(!outDir.exists()) {
            if(!outDir.mkdirs()) {
                // didn't exist, and we couldn't make it
                _logger.write(this.toString(), Logger.ERROR, "Failed to create directory: "+outDir.getPath());
                // bail out
                return;
            }
        }
        // directory has been made, check file exists
        if(!outFile.exists()) {
            try {
                outFile.createNewFile();
            } catch (IOException e) {
                _logger.write(this.toString(), Logger.ERROR, "Failed to create file: "+e);
                // bail out
                return;
            }
        }
        // file should now exist
        if(outFile.canWrite()) {
            PrintWriter out;
            try {
                out = new PrintWriter(new FileWriter(outFile));
                out.println(data);
                out.close();
            } catch (IOException e) {
                _logger.write(this.toString(), Logger.ERROR, "Failed to write file: "+e);
            }
        }
        else {
            _logger.write(this.toString(), Logger.ERROR, "File not writeable: "+outFile.getPath());
        }
    }
    
    /**
     * Iterates through dir (a directory) and deletes
     * all files and subdirectories.
     *
     * @param dir the directory to clear
     * @return true if it succeeded
     */
    private boolean deleteContents(File dir) {
        boolean success = true;
        if(dir.isDirectory()) {
            // is a directory
            File[] contents = dir.listFiles();
            for(int i=0; i < contents.length; i++) {
                File sub = contents[i];
                if(sub.isDirectory()) {
                    // lets get recursive
                    success = success & deleteContents(sub);
                }
                // it's a file or empty dir
                success = success & sub.delete();
            }
        }
        else {
            // not a directory?
            success=false;
        }
        return success;
    }
        
//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = ClientMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
//---STATIC ATTRIBUTES---

    /**
     * A reference to the single instance of this class
     */
    private static WebFeeder _instance;

}
