/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.monitors;

//---IMPORTS---
import java.util.HashMap;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This Monitor watches the Process Counts for all machines
 *
 * @author  $Author: tdb $
 * @version $Id: Process__Monitor.java,v 1.7 2003/02/05 16:43:45 tdb Exp $
 */
public class Process__Monitor extends MonitorSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.7 $";
    
    /**
     * A description of this monitor
     */
    public final String DESC = "Monitors Process Counts.";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---
    
    /**
     * Analyse a packet of data, and generate an alert if
     * necessary.
     *
     * @param packet the XMLPacket to analyse
     */
    public void analysePacket(XMLPacket packet) {

        String source = packet.getParam("packet.attributes.machine_name");
        if (!_hosts.containsKey(source)) {
            HashMap attributeRegisters = new HashMap();
            initAttributeRegsiters(source, attributeRegisters);
            _hosts.put(source, attributeRegisters);
        }

        HashMap attributeRegisters = (HashMap) _hosts.get(source);
        for(int attributeNum = 0; attributeNum < _attributes.length; attributeNum++) {
            Register reg = (Register) attributeRegisters.get(_attributes[attributeNum]);
            // find out the threshold level we're at
            String attribute = _attributes[attributeNum];
            String attributeName = _attributeNames[attributeNum];
            String currentValue = packet.getParam(attribute);
            int newThreshold = checkAttributeThreshold(currentValue, reg);
            processAlert(newThreshold, attributeName, reg, source, currentValue);
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * return the String representation of what the monitor does
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---
    
    /**
     * Checks a piece of current data, and returns the
     * threshold it breaches, if any.
     *
     * @param attributeString a String representing the current data value
     * @param reg the Register for the host
     * @return the threshold level breached, if any
     */
    private int checkAttributeThreshold(String attributeString, Register reg) {
        for(int thresholdLevel = Alert.thresholdLevels.length - 1; thresholdLevel >= 0; thresholdLevel--) {
            if (reg.getThreshold(thresholdLevel) != -1.0) {
                if(attributeString != null) {
                    try {
                        double attribute = Double.parseDouble(attributeString);
                        if (reg.getThreshold(thresholdLevel) < attribute) return thresholdLevel;
                    } catch (NumberFormatException e) {
                        // we got some duff data in the packet, but we shouldn't have
                        _logger.write(toString(), Logger.DEBUG, "possible errenous packet data, should be double value - " + attributeString);
                    }
                }
            }
        }
        return Alert.thresholdNORMAL;
    }
    
    /**
     * Initialises a HashMap of Registers with the current list
     * of attributes. This is only used if we are looking at more
     * than one distinct attribute.
     *
     * @param source the host we are looking at
     * @param attributeRegisters a HashMap to put the new Registers in
     */
    private void initAttributeRegsiters(String source, HashMap attributeRegisters) {
        for(int attributeNum = 0; attributeNum < _attributes.length; attributeNum++) {
            String attributeName = _attributes[attributeNum].substring(_attributes[attributeNum].lastIndexOf(".") + 1);
            attributeRegisters.put(_attributes[attributeNum], new Register(source, _name, attributeName));
        }
    }

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns a reference to a specific Queue for this
     * monitor. This Queue returns only the data packets
     * (based on type) that we want too look at.
     *
     * @return a reference to a Queue
     */
    protected Queue getQueue() {
        return MonitorManager.getInstance().getDataQueue();
    }

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = "Process";
    
    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring.
     */
    private HashMap _hosts = new HashMap();
    
    /**
     * An array of attributes which we will be checking.
     */
    private String[] _attributes = { "packet.processes.total", "packet.processes.cpu", "packet.processes.sleeping", "packet.processes.stopped", "packet.processes.zombie" };
    
    /**
     * An array of "nice names" for the attributes in _attributes.
     */
    private String[] _attributeNames = {"Total Processes", "on CPU Processes", "Sleeping Processes", "Stopped Processes", "Zombie Processes"};

//---STATIC ATTRIBUTES---

}
