/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filter;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.filter.*;

/**
 * A Filter Startup Class
 * A filter is an iscream component.
 *
 * @author  $Author: tdb $
 * @version $Id: FilterMain.java,v 1.39 2003/02/24 20:18:49 tdb Exp $
 */
public class FilterMain implements Component {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.39 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---
    
    /**
     * Constructs a Filter with the name given
     *
     * @param givenName the name
     */
    public FilterMain(String givenName) {
        NAME = givenName;
    }

//---PUBLIC METHODS---

    /**
     * Starts the Filter component
     */
    public void start() throws ComponentStartException {
        // get references to key objects
        _logger = _refman.getLogger();
        
        _logger.write(toString(), Logger.SYSINIT, "coming up");
        
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        
        // which input methods do we need to activate?
        // default to activating them
        boolean activateUDPReader = true;
        boolean activateCORBAReader = true;
        
        // check for UDP Reader
        try {
            int udp = Integer.parseInt(cp.getProperty("Filter." + FilterMain.NAME, "Filter.ActivateUDPReader"));
            activateUDPReader = (udp == 1);
        } catch (PropertyNotFoundException e) {
            activateUDPReader = false;
        } catch (NumberFormatException e) {
            activateUDPReader = false;
        }
        // check for CORBA Reader
        try {
            int corba = Integer.parseInt(cp.getProperty("Filter." + FilterMain.NAME, "Filter.ActivateCORBAReader"));
            activateCORBAReader = (corba == 1);
        } catch (PropertyNotFoundException e) {
            activateCORBAReader = false;
        } catch (NumberFormatException e) {
            activateCORBAReader = false;
        }
        
        // need to use the Queue later on
        Queue queue;
        
        // there's little point starting a Queue and a FilterThread
        // if nothing is going to be giving us any data
        if(activateUDPReader || activateCORBAReader) {
            // see if this Queue needs a size limit
            try {
                int queueSizeLimit = Integer.parseInt(cp.getProperty("Filter." + FilterMain.NAME, "Queue.SizeLimit"));
                String queueRemoveAlgorithm = cp.getProperty("Filter." + FilterMain.NAME, "Queue.RemoveAlgorithm");
                int algorithm = StringUtils.getStringPos(queueRemoveAlgorithm, Queue.algorithms);
                if(algorithm != -1) {
                    _logger.write(toString(), Logger.DEBUG, "Starting Queue with size limit of "+queueSizeLimit+", using remove algorithm "+queueRemoveAlgorithm);
                    // we have valid values, so lets start it.
                    queue = new Queue(queueSizeLimit, algorithm);
                }
                else {
                    _logger.write(toString(), Logger.WARNING, "Bad Queue Algorithm configuration, not known: "+queueRemoveAlgorithm);
                    // just don't activate a limit
                    queue = new Queue();
                }
                
            } catch (PropertyNotFoundException e) {
                _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
                // just don't activate a limit
                queue = new Queue();
            } catch (NumberFormatException e) {
                _logger.write(toString(), Logger.WARNING, "Bad Queue SizeLimit configuration: "+e);
                // just don't activate a limit
                queue = new Queue();
            }
            
            // startup a monitor on this queue
            try {
                // try to get the interval, if this fails, we won't start up the monitor
                int queueMonitorInterval = Integer.parseInt(cp.getProperty("Filter." + FilterMain.NAME, "Queue.MonitorInterval"));
                String queueName = NAME + " Filter";
                queue.startMonitor(queueMonitorInterval*1000, queueName);
            } catch (PropertyNotFoundException e) {
                _logger.write(toString(), Logger.WARNING, "failed to find queue monitor config, disabling. " + e);
            }
            
            // Start a filter thread
            _logger.write(toString(), Logger.SYSINIT, "starting Filter Thread / Queue consumer");
            FilterThread filterThread = new FilterThread(queue);
            filterThread.start();
        }
        else {
            // it's pointless carrying on really...
            throw new ComponentStartException("Can't start Filter without any inbound data feeds");            
        }
        
        // the corba listener needs these to be set, so lets
        // make them something obviously invalid initially
        int UDPListenPort = -1;
        
        // UDP Reader start (for inbound host data)
        if(activateUDPReader) {
            try {
                // get the port number from the configuration
                UDPListenPort = Integer.parseInt(cp.getProperty("Filter." + FilterMain.NAME, "Filter.UDPListenPort"));
                // start the UDPReader
                _logger.write(toString(), Logger.SYSINIT, "starting Filter UDP listener");
                UDPReader udpReader = new UDPReader(UDPListenPort, queue);
                udpReader.start();
            } catch (PropertyNotFoundException e) {
                _logger.write(toString(), Logger.WARNING, "Unable to start UDPReader due to missing configuration: " + e);
            } catch (NumberFormatException e) {
                _logger.write(toString(), Logger.WARNING, "Unable to start UDPReader due to invalid configuration: " + e);
            }
        }
        
        // FilterServant start (for inbound child filter data)
        if(activateCORBAReader) {
            // start the FilterServant
            _logger.write(toString(), Logger.SYSINIT, "starting Servant to listen for downstream filters");
            FilterServant filterServant = new FilterServant(queue);
            _refman.bindToOrb(filterServant, "iscream.Filter\\." + FilterMain.NAME);
        }
        
        // FilterInfoServant start (to provide filter information to the server)
        _logger.write(toString(), Logger.SYSINIT, "starting Servant to provide filter information");
        FilterInfoServant filterInfoServant = new FilterInfoServant(UDPListenPort);
        _refman.bindToOrb(filterInfoServant, "iscream.FilterInfo\\." + FilterMain.NAME);
        
        _logger.write(toString(), Logger.SYSINIT, "started");
    }
    
    /**
     * Does a dependency check. Used mainly at startup to
     * see if the required dependencies (components) are up
     * and running.
     *
     * @return a boolean value, true if the depdencies are satisfied
     */
    public boolean depCheck() {
        try {
            org.omg.CORBA.Object obj;
            // first check the ConfigurationManager is alive
            obj = _refman.getCORBARef("iscream.ConfigurationManager");
            // then suss out our parent filter
            ConfigurationProxy cp = ConfigurationProxy.getInstance();
            String parentFilterName = cp.getProperty("Filter." + FilterMain.NAME, "Filter.parentFilter");
            // finally check the parent filter is alive
            obj = _refman.getCORBARef("iscream.Filter\\." + parentFilterName);
        } catch(ComponentCORBAException e) {
            System.err.println(toString() + ": Dependency Failure: "+e);
            return false;
        } catch(PropertyNotFoundException e) {
            System.err.println(toString() + ": Unable to obtain configuration: "+e);
            return false;
        }
        // dependency check suceeded
        return true;
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            NAME,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger;

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();

//---STATIC ATTRIBUTES---

    /**
     * The friendly name for this component, used by
     * all related classes.
     * This is set from the configuration.
     */
    public static String NAME;

}
