/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filter;

//---IMPORTS---
import java.io.*;
import java.util.*;

import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;

/**
 * This class setups up and manages Plugins for performing service
 * checks.  A list of plugins to use is specified in the configuration,
 * and these are all loaded for a specific host when asked.
 * It asks the configuration manager for references to ServiceChecks to
 * perform, thus ensuring only one instance of a service check is loaded
 * for all hosts.
 *
 * @author  $Author: tdb $
 * @version $Id: PluginServiceCheckPipeline.java,v 1.10 2003/02/05 16:43:47 tdb Exp $
 */
class PluginServiceCheckPipeline {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.10 $";
    
    /**
     * file name suffix for plugin classes:
     */
    private final String _suffix = "__ServiceCheck";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates a new pipeline
     *
     * @param hostname the host this pipeline is for
     * @param manager the manager to obtain services checks from
     */
    public PluginServiceCheckPipeline(String hostname, PluginServiceCheckManager manager){
        _logger.write(toString(), Logger.SYSINIT, "Created.");
        _hostname = hostname;
        _manager = manager;
        _pipeline = buildPipeline();
        _created = System.currentTimeMillis();
    }

//---PUBLIC METHODS---

    /**
     * Instructs this pipeline to execute
     *
     * @return the XML generated by all the service checks
     */
    public String runPipeline(){
        // if our config has changed, rebuild the pipeline
        if (_refman.getCM().isModified(_fileList, _lastModified)) {
            _pipeline = buildPipeline();
        }
        // for each service check in the pipeline...
        String xml = "";
        Iterator i = _pipeline.iterator();
        while (i.hasNext()){
            PluginServiceCheck plugin = (PluginServiceCheck) i.next();
            xml += plugin.runServiceCheck(_hostname);
        }
        return xml;
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    /**
     * Builds a pipeline of service checks to be run.
     * This method is called to read in the service checks
     * for a particular host from the configurations.  It then
     * builds a linked list of theses checks and then returns
     * them.
     *
     * @param hostname the hostname of the host to build the pipeline for
     * @return the pipeline
     */
    private LinkedList buildPipeline() {
        _logger.write(toString(), Logger.SYSMSG, "Creating plugin pipeline for service checks for - " + _hostname);
        
        String pluginsPackage, pluginsList;
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        String configName = "Host."+_hostname;
        
        _lastModified = cp.getLastModified(configName);
        _fileList = cp.getFileList(configName);
        
        try {
            pluginsPackage = cp.getProperty(configName, "Host.serviceChecksPackage");
            pluginsList = cp.getProperty(configName, "Host.serviceChecks");
        } catch(PropertyNotFoundException e) {
            pluginsPackage = "";
            pluginsList = "";
        }
        
        LinkedList pipeline = new LinkedList();
        _logger.write(toString(), Logger.DEBUG, "building pipeline of - " + pluginsList);        
        StringTokenizer st = new StringTokenizer(pluginsList, ";");
        while(st.hasMoreTokens()) {
            String className = pluginsPackage + "." + st.nextToken() + _suffix;
            pipeline.add(_manager.getServiceCheck(className));
        }
        return pipeline;
    }

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Returns the system time that this
     * instance was created at.  Used for 
     * management of pipelines to determine
     * when its use is finished.
     *
     * @return the time of creation
     */
    public long getCreated() {
        return _created;
    }

//---ATTRIBUTES---
    
    /**
     * The name of the host this pipleline is for
     */
    private String _hostname;
    
    /**
     * Holds the current pipeline of plugins
     */
    private LinkedList _pipeline;
    
    /**
     * When this pipleine was created
     */
    private long _created;
    
    /**
     * When the current pipeline configuration was last modified
     */
    private long _lastModified;
    
    /**
     * The list of files used in the configuration
     */
    private String _fileList;
    
    /**
     * The plugin manager from which we can 
     * obtain references to service checks
     */
    private PluginServiceCheckManager _manager;
 
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = FilterMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();

//---STATIC ATTRIBUTES---

}
